#ifndef AUBO_SCOPE_UNIT_CONVERTOR_H
#define AUBO_SCOPE_UNIT_CONVERTOR_H

#include <math.h>

#include <QString>
#include <aubo_caps/domain/system_api.h>

namespace arcs {
namespace aubo_scope {

template <typename T>
class MeasureBase
{
public:
    MeasureBase(double value = 0.) : value_(value) {}

    bool operator==(const MeasureBase &other) { return value_ == other.value_; }

    QString toString(UnitType unit)
    {
        return QString::number(value_) + " " + T::suffix(unit);
    }

    operator double() { return value_; }

protected:
    double value_;
};

class Length : public MeasureBase<Length>
{
public:
    Length(double value = 0.) : MeasureBase(value) {}
    Length(double value, UnitType unit)
    {
        switch (unit) {
        case UnitType::METRIC:
            value_ = value / 1000.;
            break;
        case UnitType::IMPERIAL:
            value_ = value * 25.4 / 1000.;
            break;
        }
    }

    static QString suffix(UnitType unit)
    {
        switch (unit) {
        case UnitType::METRIC:
            return "mm";
            break;
        case UnitType::IMPERIAL:
            return "in";
            break;
        }
    }

    double getAs(UnitType unit)
    {
        switch (unit) {
        case UnitType::METRIC:
            return value_ * 1000.;
            break;
        case UnitType::IMPERIAL:
            return value_ * 1000. / 25.4;
            break;
        }
    }
};

class Speed : public MeasureBase<Speed>
{
public:
    Speed(double value = 0.) : MeasureBase(value) {}
    Speed(double value, UnitType unit)
    {
        switch (unit) {
        case UnitType::METRIC:
            value_ = value / 1000.;
            break;
        case UnitType::IMPERIAL:
            value_ = value * 25.4 / 1000.;
            break;
        }
    }

    static QString suffix(UnitType unit)
    {
        switch (unit) {
        case UnitType::METRIC:
            return "mm/s";
            break;
        case UnitType::IMPERIAL:
            return "in/s";
            break;
        }
    }

    double getAs(UnitType unit)
    {
        switch (unit) {
        case UnitType::METRIC:
            return value_ * 1000.;
            break;
        case UnitType::IMPERIAL:
            return value_ * 1000. / 25.4;
            break;
        }
    }
};

class Acceleration : public MeasureBase<Acceleration>
{
public:
    Acceleration(double value = 0.) : MeasureBase(value) {}
    Acceleration(double value, UnitType unit)
    {
        switch (unit) {
        case UnitType::METRIC:
            value_ = value / 1000.;
            break;
        case UnitType::IMPERIAL:
            value_ = value / 25.4 / 1000.;
            break;
        }
    }

    static QString suffix(UnitType unit)
    {
        switch (unit) {
        case UnitType::METRIC:
            return "mm/s²";
            break;
        case UnitType::IMPERIAL:
            return "in/s²";
            break;
        }
    }

    double getAs(UnitType unit)
    {
        switch (unit) {
        case UnitType::METRIC:
            return value_ * 1000.;
            break;
        case UnitType::IMPERIAL:
            return value_ * 1000. * 25.4;
            break;
        }
        return value_;
    }
};

class Angle : public MeasureBase<Angle>
{
public:
    Angle(double value = 0.) : MeasureBase(value) {}
    Angle(double value, UnitType unit)
    {
        switch (unit) {
        case UnitType::METRIC:
            break;
        case UnitType::IMPERIAL:
            break;
        }
        value_ = value / 180. * M_PI;
    }

    static QString suffix(UnitType unit)
    {
        switch (unit) {
        case UnitType::METRIC:
            return "°";
            break;
        case UnitType::IMPERIAL:
            return "°";
            break;
        }
    }

    double getAs(UnitType unit)
    {
        switch (unit) {
        case UnitType::METRIC:
            break;
        case UnitType::IMPERIAL:
            break;
        }
        return value_ * 180. / M_PI;
    }
};

class Orientation : public MeasureBase<Orientation>
{
public:
    Orientation(double value = 0.) : MeasureBase(value) {}
    Orientation(double value, UnitType unit)
    {
        switch (unit) {
        case UnitType::METRIC:
            break;
        case UnitType::IMPERIAL:
            break;
        }
        value_ = value;
    }

    static QString suffix(UnitType unit)
    {
        switch (unit) {
        case UnitType::METRIC:
            return "rad";
            break;
        case UnitType::IMPERIAL:
            return "rad";
            break;
        }
    }

    double getAs(UnitType unit)
    {
        switch (unit) {
        case UnitType::METRIC:
            break;
        case UnitType::IMPERIAL:
            break;
        }
        return value_;
    }
};

class AngularAcceleration : public MeasureBase<AngularAcceleration>
{
public:
    AngularAcceleration(double value = 0.) : MeasureBase(value) {}
    AngularAcceleration(double value, UnitType unit)
    {
        switch (unit) {
        case UnitType::METRIC:
            break;
        case UnitType::IMPERIAL:
            break;
        }
        value_ = value / 180. * M_PI;
    }

    static QString suffix(UnitType unit)
    {
        switch (unit) {
        case UnitType::METRIC:
            return "°/s²";
            break;
        case UnitType::IMPERIAL:
            return "°/s²";
            break;
        }
    }

    double getAs(UnitType unit)
    {
        switch (unit) {
        case UnitType::METRIC:
            break;
        case UnitType::IMPERIAL:
            break;
        }
        return value_ * 180. / M_PI;
    }
};

class AngularSpeed : public MeasureBase<AngularSpeed>
{
public:
    AngularSpeed(double value = 0.) : MeasureBase(value) {}
    AngularSpeed(double value, UnitType unit) { value_ = value / 180. * M_PI; }

    static QString suffix(UnitType unit)
    {
        switch (unit) {
        case UnitType::METRIC:
            return "°/s";
            break;
        case UnitType::IMPERIAL:
            return "°/s";
            break;
        }
    }

    double getAs(UnitType unit)
    {
        switch (unit) {
        case UnitType::METRIC:
            break;
        case UnitType::IMPERIAL:
            break;
        }
        return value_ * 180. / M_PI;
    }
};

class Current : public MeasureBase<Current>
{
public:
    Current(double value = 0.) : MeasureBase(value) {}
    Current(double value, UnitType unit) { value_ = value; }

    static QString suffix(UnitType unit)
    {
        switch (unit) {
        case UnitType::METRIC:
            return "A";
            break;
        case UnitType::IMPERIAL:
            return "A";
            break;
        }
    }

    double getAs(UnitType unit)
    {
        switch (unit) {
        case UnitType::METRIC:
            break;
        case UnitType::IMPERIAL:
            break;
        }
        return value_;
    }
};

class Voltage : public MeasureBase<Voltage>
{
public:
    Voltage(double value = 0.) : MeasureBase(value) {}
    Voltage(double value, UnitType unit) { value_ = value; }

    static QString suffix(UnitType unit)
    {
        switch (unit) {
        case UnitType::METRIC:
            return "V";
            break;
        case UnitType::IMPERIAL:
            return "V";
            break;
        }
    }

    double getAs(UnitType unit)
    {
        switch (unit) {
        case UnitType::METRIC:
            break;
        case UnitType::IMPERIAL:
            break;
        }
        return value_;
    }
};

class Force : public MeasureBase<Force>
{
public:
    Force(double value = 0.) : MeasureBase(value) {}
    Force(double value, UnitType unit) {}

    static QString suffix(UnitType unit)
    {
        switch (unit) {
        case UnitType::METRIC:
            return "N";
            break;
        case UnitType::IMPERIAL:
            return "lbf";
            break;
        }
    }

    double getAs(UnitType unit)
    {
        switch (unit) {
        case UnitType::METRIC:
            break;
        case UnitType::IMPERIAL:
            break;
        }
        return value_;
    }
};

class Mass : public MeasureBase<Mass>
{
public:
    Mass(double value = 0.) : MeasureBase(value) {}
    Mass(double value, UnitType unit)
    {
        switch (unit) {
        case UnitType::METRIC:
            value_ = value;
            break;
        case UnitType::IMPERIAL:
            value_ = value * 0.45359237;
            break;
        }
    }

    static QString suffix(UnitType unit)
    {
        switch (unit) {
        case UnitType::METRIC:
            return "Kg";
            break;
        case UnitType::IMPERIAL:
            return "lb";
            break;
        }
    }

    double getAs(UnitType unit)
    {
        switch (unit) {
        case UnitType::METRIC:
            return value_;
            break;
        case UnitType::IMPERIAL:
            return value_ / 0.45359237;
            break;
        }
        return value_;
    }
};

class MomentOfInertia : public MeasureBase<MomentOfInertia>
{
public:
    MomentOfInertia(double value = 0.) : MeasureBase(value) {}
    MomentOfInertia(double value, UnitType unit) {}

    static QString suffix(UnitType unit)
    {
        switch (unit) {
        case UnitType::METRIC:
            return "Kg m/s";
            break;
        case UnitType::IMPERIAL:
            return "lb in/s";
            break;
        }
    }

    double getAs(UnitType unit)
    {
        switch (unit) {
        case UnitType::METRIC:
            break;
        case UnitType::IMPERIAL:
            break;
        }
        return value_;
    }
};

class Moment : public MeasureBase<Moment>
{
public:
    Moment(double value = 0.) : MeasureBase(value) {}
    Moment(double value, UnitType unit) {}

    static QString suffix(UnitType unit)
    {
        switch (unit) {
        case UnitType::METRIC:
            return "Kg m/s";
            break;
        case UnitType::IMPERIAL:
            return "lb in/s";
            break;
        }
    }

    double getAs(UnitType unit)
    {
        switch (unit) {
        case UnitType::METRIC:
            break;
        case UnitType::IMPERIAL:
            break;
        }
        return value_;
    }
};

class Pressure : public MeasureBase<Pressure>
{
public:
    Pressure(double value = 0.) : MeasureBase(value) {}
    Pressure(double value, UnitType unit) {}

    static QString suffix(UnitType unit)
    {
        switch (unit) {
        case UnitType::METRIC:
            return "Kg m/s";
            break;
        case UnitType::IMPERIAL:
            return "lb in/s";
            break;
        }
    }

    double getAs(UnitType unit)
    {
        switch (unit) {
        case UnitType::METRIC:
            break;
        case UnitType::IMPERIAL:
            break;
        }
        return value_;
    }
};

class Torque : public MeasureBase<Torque>
{
public:
    Torque(double value = 0.) : MeasureBase(value) {}
    Torque(double value, UnitType unit) {}

    static QString suffix(UnitType unit)
    {
        switch (unit) {
        case UnitType::METRIC:
            return "Kg m/s";
            break;
        case UnitType::IMPERIAL:
            return "lb in/s";
            break;
        }
    }

    double getAs(UnitType unit)
    {
        switch (unit) {
        case UnitType::METRIC:
            break;
        case UnitType::IMPERIAL:
            break;
        }
        return value_;
    }
};

class Time : public MeasureBase<Time>
{
public:
    Time(double value = 0.) : MeasureBase(value) {}
    Time(double value, UnitType unit) {}

    static QString suffix(UnitType unit)
    {
        switch (unit) {
        case UnitType::METRIC:
            return "s";
            break;
        case UnitType::IMPERIAL:
            return "s";
            break;
        }
    }

    double getAs(UnitType unit)
    {
        switch (unit) {
        case UnitType::METRIC:
            break;
        case UnitType::IMPERIAL:
            break;
        }
        return value_;
    }
};

class TinyTime : public MeasureBase<Time>
{
public:
    TinyTime(double value = 0.) : MeasureBase(value) {}
    TinyTime(double value, UnitType unit)
    {
        switch (unit) {
        case UnitType::METRIC:
            break;
        case UnitType::IMPERIAL:
            break;
        }

        value_ = value / 1000.;
    }

    static QString suffix(UnitType unit)
    {
        switch (unit) {
        case UnitType::METRIC:
            return "ms";
            break;
        case UnitType::IMPERIAL:
            return "ms";
            break;
        }
    }

    double getAs(UnitType unit)
    {
        switch (unit) {
        case UnitType::METRIC:
            break;
        case UnitType::IMPERIAL:
            break;
        }
        return value_ * 1000.;
    }
};

class Power : public MeasureBase<Power>
{
public:
    Power(double value = 0.) : MeasureBase(value) {}
    Power(double value, UnitType unit) {}

    static QString suffix(UnitType unit)
    {
        switch (unit) {
        case UnitType::METRIC:
            return "W";
            break;
        case UnitType::IMPERIAL:
            return "W";
            break;
        }
    }

    double getAs(UnitType unit)
    {
        switch (unit) {
        case UnitType::METRIC:
            break;
        case UnitType::IMPERIAL:
            break;
        }
        return value_;
    }
};

} // namespace aubo_scope
} // namespace arcs
#endif
