#ifndef AUBO_SCOPE_KEYBOARD_EXPRESSION_INPUT_H
#define AUBO_SCOPE_KEYBOARD_EXPRESSION_INPUT_H

#include <memory>
#include <functional>

#include <aubo_caps/class_forward.h>
#include <aubo_caps/domain/user_interface/keyboard_input_callback.h>
#include <aubo_caps/domain/user_interface/input_validator.h>

class QWidget;

namespace arcs {
namespace aubo_scope {

ARCS_CLASS_FORWARD(Expression);
ARCS_CLASS_FORWARD(KeyboardExpressionInput);

/**
 * Interface representing the virtual keyboard used for configuring it and
 * requesting it to be displayed in PolyScope.
 */
class ARCS_ABI_EXPORT KeyboardExpressionInput
{
public:
    KeyboardExpressionInput(KeyboardExpressionInput &f);
    KeyboardExpressionInput(KeyboardExpressionInput &&f);
    virtual ~KeyboardExpressionInput();

    /**
     * <p>
     * Sets an input validator for the virtual keyboard. This validator will be
     * used to validate the input entered by the user. Some standard validators
     * are available in {@link
     * domain.userinteraction.inputvalidation.InputValidationFactory}.
     * </p>
     *
     * <p>
     * Specifying an input validator is recommended, but optional.
     * </p>
     *
     * In case {@link InputValidator#isValid(Object)} (in this case parameter is
     * {@link String}) returns <code>false</code>, the accept button on the
     * virtual keyboard will be disabled and the error message from {@link
     * InputValidator#getMessage(Object)} (parameter is {@link String}) will be
     * shown.
     *
     * @param validator the validator to be used to validate keyboard user
     * input.
     * @return this KeyboardTextInput object.
     * @throws NullPointerException in case the validator is <code>null</code>.
     */
    void setErrorValidator(
        const std::shared_ptr<InputValidator<ExpressionPtr>> &validator);

    /**
     * Sets the text which will be displayed when the virtual keyboard is shown.
     *
     * @param initialExpression the initial text for the virtual keyboard.
     * @return this KeyboardTextInput object.
     */
    void setInitialValue(const ExpressionPtr &initialExpression);

    /**
     * This method requests the virtual keyboard to be shown.
     *
     * @param component the JTextField that will be used to define the screen
     * position of the virtual keyboard. The component must be visible on the
     * screen at the time of showing the keypad.
     * @param callback  the callback to be used when the user exits the keyboard
     * by accepting or canceling the input.
     * @throws NullPointerException in case any parameter is <code>null</code>.
     */
    void show(QWidget *component,
              const std::function<void(bool, const ExpressionPtr &)> &callback);

private:
    friend class DataSwitch;
    KeyboardExpressionInput();
    void *d_{ nullptr };
};
} // namespace aubo_scope
} // namespace arcs

#endif
