#ifndef AUBO_SCOPE_KEYBOARD_INPUT_FACTORY_H
#define AUBO_SCOPE_KEYBOARD_INPUT_FACTORY_H

#include <memory>
#include <string>
#include <aubo_caps/class_forward.h>
#include <aubo_caps/domain/user_interface/keyboard_number_input.h>
#include <aubo_caps/domain/user_interface/keyboard_text_input.h>
#include <aubo_caps/domain/user_interface/keyboard_expression_input.h>

namespace arcs {
namespace aubo_scope {
ARCS_CLASS_FORWARD(KeyboardInputFactory);

/**
 * <p>
 * Factory for creating keyboard inputs which are used to configure the virtual
 * keyboard/keypad and to request it to be displayed for a Swing GUI component.
 * The typical Swing component most commonly used is a {@link
 * javax.swing.JTextField}.
 * </p>
 *
 * NOTE: This functionality is only relevant for AuboCap nodes with a
 * Swing-based user interface (see {@link ProgramNodeService}, {@link
 * InstallationNodeService} and ToolbarService}).
 */
class ARCS_ABI_EXPORT KeyboardInputFactory
{
public:
    KeyboardInputFactory(KeyboardInputFactory &f);
    KeyboardInputFactory(KeyboardInputFactory &&f);
    virtual ~KeyboardInputFactory();

    /**
     * <p>
     * Creates a standard virtual keyboard input.
     * </p>
     *
     * If no input validator is specified, all strings are accepted.
     *
     * @return new instance of KeyboardTextInput.
     */
    KeyboardTextInputPtr createStandardKeyboardInput();

    /**
     * <p>
     * Creates a standard virtual keyboard input for naming.
     * </p>
     *
     * If no input validator is specified, all strings are accepted.
     *
     * @return new instance of KeyboardTextInput.
     */
    KeyboardTextInputPtr createStringKeyboardInput();

    /**
     * <p>
     * Creates a expression virtual keyboard input for expression.
     * </p>
     *
     * If no input validator is specified, all strings are accepted.
     *
     * @return new instance of KeyboardTextInput.
     */
    KeyboardExpressionInputPtr createExpressionKeyboardInput();

    /**
     * <p>
     * Creates a standard virtual keyboard input accepting password string/text
     * input. When the end user types, asterisks are shown instead of the
     * original characters.
     * </p>
     *
     * If no input validator is specified, all strings are accepted.
     *
     * @return new instance of KeyboardTextInput.
     */
    KeyboardTextInputPtr createPasswordKeyboardInput();

    /**
     * Creates a virtual keyboard input accepting IPv4 addresses. The keyboards
     * accepts any IPv4 address (ensuring the correct format) and returns it as
     * a string.
     *
     * @return new instance of <code>KeyboardTextInput</code>.
     */
    std::shared_ptr<KeyboardIntInput> createIPAddressKeyboardInput();

    /**
     * <p>
     * Creates a virtual numeric keypad input accepting doubles
     * </p>
     *
     * If no input validator is specified, all double values are accepted.
     *
     * @return new instance of <code>KeyboardNumberInput<Double></code>.
     */
    std::shared_ptr<KeyboardDoubleInput> createDoubleKeypadInput();

    /**
     * <p>
     * Creates a virtual numeric keypad input accepting positive doubles. The
     * minus ('-') button on the keypad will be disabled.
     * </p>
     *
     * If no input validator is specified, all positive double values are
     * accepted.
     *
     * @return new instance of <code>KeyboardNumberInput<Double></code>.
     */
    std::shared_ptr<KeyboardDoubleInput> createPositiveDoubleKeypadInput();

    /**
     * <p>
     * Creates a virtual numeric keypad input accepting integers. The dot ('.')
     * button on the keypad will be disabled.
     * </p>
     *
     * If no input validator is specified, all integer values are accepted.
     *
     * @return new instance of <code>KeyboardNumberInput<Integer>}.
     */
    std::shared_ptr<KeyboardIntInput> createIntegerKeypadInput();

    /**
     * <p>
     * Creates a virtual numeric keypad input accepting positive integers. The
     * minus ('-') and the dot ('.') buttons on the keypad will be disabled.
     * </p>
     *
     * If no input validator is specified, all positive integer values are
     * accepted.
     *
     * @return new instance of <code>KeyboardNumberInput<Integer></code>.
     */
    std::shared_ptr<KeyboardUIntInput> createPositiveIntegerKeypadInput();

private:
    friend class DataSwitch;
    KeyboardInputFactory();
    void *d_{ nullptr };
};
} // namespace aubo_scope
} // namespace arcs

#endif
